@@ -1,40 +1,66 @@
+# SPDX-License-Identifier: GPL-2.0-only
 # This file is part of Scapy
-# See http://www.secdev.org/projects/scapy for more information
+# See https://scapy.net/ for more information
 # Copyright (C) Philippe Biondi <phil@secdev.org>
-# This program is published under a GPLv2 license
+
 # Netflow V5 appended by spaceB0x and Guillaume Valadon
-# Netflow V9/10 appended ny Gabriel Potter
+# Netflow V9/10 appended by Gabriel Potter
 
 """
 Cisco NetFlow protocol v1, v5, v9 and v10 (IPFix)
 
-HowTo dissect NetflowV9/10 (IPFix) packets:
+HowTo dissect NetflowV9/10 (IPFix) packets
 
 # From a pcap / list of packets
 
-Using sniff and sessions:
->>> sniff(offline=open("my_great_pcap.pcap", "rb"), session=NetflowSession)
+Using sniff and sessions::
+
+    >>> sniff(offline=open("my_great_pcap.pcap", "rb"), session=NetflowSession)
 
 Using the netflowv9_defragment/ipfix_defragment commands:
+
 - get a list of packets containing NetflowV9/10 packets
 - call `netflowv9_defragment(plist)` to defragment the list
 
 (ipfix_defragment is an alias for netflowv9_defragment)
 
-# Live / on-the-flow / other: use NetflowSession
->>> sniff(session=NetflowSession, prn=[...])
+# Live / on-the-flow / other: use NetflowSession::
+
+    >>> sniff(session=NetflowSession, prn=[...])
+
 """
 
+import socket
 import struct
 
 from scapy.config import conf
 from scapy.data import IP_PROTOS
-from scapy.error import warning
-from scapy.fields import ByteEnumField, ByteField, Field, FieldLenField, \
-    FlagsField, IPField, IntField, MACField, \
-    PacketListField, PadField, SecondsIntField, ShortEnumField, ShortField, \
-    StrField, StrFixedLenField, ThreeBytesField, UTCTimeField, XByteField, \
-    XShortField, LongField
+from scapy.error import warning, Scapy_Exception
+from scapy.fields import (
+    BitEnumField,
+    BitField,
+    ByteEnumField,
+    ByteField,
+    ConditionalField,
+    Field,
+    FieldLenField,
+    FlagsField,
+    IPField,
+    IntField,
+    LongField,
+    MACField,
+    PacketListField,
+    SecondsIntField,
+    ShortEnumField,
+    ShortField,
+    StrField,
+    StrFixedLenField,
+    StrLenField,
+    ThreeBytesField,
+    UTCTimeField,
+    XByteField,
+    XShortField,
+)
 from scapy.packet import Packet, bind_layers, bind_bottom_up
 from scapy.plist import PacketList
 from scapy.sessions import IPSession, DefaultSession
@@ -61,11 +87,17 @@ bind_layers(UDP, NetflowHeader, dport=2055, sport=2055)
 
 class NetflowHeaderV1(Packet):
     name = "Netflow Header v1"
-    fields_desc = [ShortField("count", 0),
+    fields_desc = [ShortField("count", None),
                    IntField("sysUptime", 0),
                    UTCTimeField("unixSecs", 0),
                    UTCTimeField("unixNanoSeconds", 0, use_nano=True)]
 
+    def post_build(self, pkt, pay):
+        if self.count is None:
+            count = len(self.layers()) - 1
+            pkt = struct.pack("!H", count) + pkt[2:]
+        return pkt + pay
+
 
 class NetflowRecordV1(Packet):
     name = "Netflow Record v1"
@@ -99,7 +131,7 @@ bind_layers(NetflowRecordV1, NetflowRecordV1)
 
 class NetflowHeaderV5(Packet):
     name = "Netflow Header v5"
-    fields_desc = [ShortField("count", 0),
+    fields_desc = [ShortField("count", None),
                    IntField("sysUptime", 0),
                    UTCTimeField("unixSecs", 0),
                    UTCTimeField("unixNanoSeconds", 0, use_nano=True),
@@ -108,6 +140,12 @@ class NetflowHeaderV5(Packet):
                    ByteField("engineID", 0),
                    ShortField("samplingInterval", 0)]
 
+    def post_build(self, pkt, pay):
+        if self.count is None:
+            count = len(self.layers()) - 1
+            pkt = struct.pack("!H", count) + pkt[2:]
+        return pkt + pay
+
 
 class NetflowRecordV5(Packet):
     name = "Netflow Record v5"
@@ -124,7 +162,7 @@ class NetflowRecordV5(Packet):
                    ShortField("dstport", 0),
                    ByteField("pad1", 0),
                    FlagsField("tcpFlags", 0x2, 8, "FSRPAUEC"),
-                   ByteEnumField("prot", IP_PROTOS["tcp"], IP_PROTOS),
+                   ByteEnumField("prot", socket.IPPROTO_TCP, IP_PROTOS),
                    ByteField("tos", 0),
                    ShortField("src_as", 0),
                    ShortField("dst_as", 0),
@@ -1240,17 +1278,30 @@ class NetflowHeaderV10(Packet):
                    IntField("flowSequence", 0),
                    IntField("ObservationDomainID", 0)]
 
+    def post_build(self, pkt, pay):
+        if self.length is None:
+            length = len(pkt) + len(pay)
+            pkt = struct.pack("!H", length) + pkt[2:]
+        return pkt + pay
+
 
 class NetflowTemplateFieldV9(Packet):
     name = "Netflow Flowset Template Field V9/10"
-    fields_desc = [ShortEnumField("fieldType", None,
-                                  NetflowV910TemplateFieldTypes),
-                   ShortField("fieldLength", 0)]
+    fields_desc = [BitField("enterpriseBit", 0, 1),
+                   BitEnumField("fieldType", None, 15,
+                                NetflowV910TemplateFieldTypes),
+                   ShortField("fieldLength", None),
+                   ConditionalField(IntField("enterpriseNumber", 0),
+                                    lambda p: p.enterpriseBit)]
 
     def __init__(self, *args, **kwargs):
         Packet.__init__(self, *args, **kwargs)
-        if self.fieldType is not None and not self.fieldLength and self.fieldType in NetflowV9TemplateFieldDefaultLengths:  # noqa: E501
-            self.fieldLength = NetflowV9TemplateFieldDefaultLengths[self.fieldType]  # noqa: E501
+        if (self.fieldType is not None and
+                self.fieldLength is None and
+                self.fieldType in NetflowV9TemplateFieldDefaultLengths):
+            self.fieldLength = NetflowV9TemplateFieldDefaultLengths[
+                self.fieldType
+            ]
 
     def default_payload_class(self, p):
         return conf.padding_layer
@@ -1318,6 +1369,34 @@ def _GenNetflowRecordV9(cls, lengths_list):
     return NetflowRecordV9I
 
 
+def GetNetflowRecordV9(flowset, templateID=None):
+    """
+    Get a NetflowRecordV9/10 for a specific NetflowFlowsetV9/10.
+
+    Have a look at the online doc for examples.
+    """
+    definitions = {}
+    for ntv9 in flowset.templates:
+        llist = []
+        for tmpl in ntv9.template_fields:
+            llist.append((tmpl.fieldLength, tmpl.fieldType))
+        if llist:
+            cls = _GenNetflowRecordV9(NetflowRecordV9, llist)
+            definitions[ntv9.templateID] = cls
+    if not definitions:
+        raise Scapy_Exception(
+            "No template IDs detected"
+        )
+    if len(definitions) > 1:
+        if templateID is None:
+            raise Scapy_Exception(
+                "Multiple possible templates ! Specify templateID=.."
+            )
+        return definitions[templateID]
+    else:
+        return list(definitions.values())[0]
+
+
 class NetflowRecordV9(Packet):
     name = "Netflow DataFlowset Record V9/10"
     fields_desc = [StrField("fieldValue", "")]
@@ -1329,14 +1408,11 @@ class NetflowRecordV9(Packet):
 class NetflowDataflowsetV9(Packet):
     name = "Netflow DataFlowSet V9/10"
     fields_desc = [ShortField("templateID", 255),
-                   FieldLenField("length", None, length_of="records",
-                                 adjust=lambda pkt, x: x + 4 + (-x % 4)),
-                   PadField(
-                       PacketListField(
-                           "records", [],
-                           NetflowRecordV9,
-                           length_from=lambda pkt: pkt.length - 4
-                       ), 4, padwith=b"\x00")]
+                   ShortField("length", None),
+                   PacketListField(
+                       "records", [],
+                       NetflowRecordV9,
+                       length_from=lambda pkt: pkt.length - 4)]
 
     @classmethod
     def dispatch_hook(cls, _pkt=None, *args, **kargs):
@@ -1354,6 +1430,15 @@ class NetflowDataflowsetV9(Packet):
                 return NetflowOptionsFlowset10
         return cls
 
+    def post_build(self, pkt, pay):
+        if self.length is None:
+            # Padding is optional, let's apply it on build
+            length = len(pkt)
+            pad = (-length) % 4
+            pkt = pkt[:2] + struct.pack("!H", length + pad) + pkt[4:]
+            pkt += b"\x00" * pad
+        return pkt + pay
+
 
 def _netflowv9_defragment_packet(pkt, definitions, definitions_opts, ignored):
     """Used internally to process a single packet during defragmenting"""
@@ -1408,49 +1493,56 @@ def _netflowv9_defragment_packet(pkt, definitions, definitions_opts, ignored):
             current = current.payload
     # Dissect flowsets
     if NetflowDataflowsetV9 in pkt:
-        datafl = pkt[NetflowDataflowsetV9]
-        tid = datafl.templateID
-        if tid not in definitions and tid not in definitions_opts:
-            ignored.add(tid)
-            return
-        # All data is stored in one record, awaiting to be split
-        # If fieldValue is available, the record has not been
-        # defragmented: pop it
-        try:
-            data = datafl.records[0].fieldValue
-            datafl.records.pop(0)
-        except (IndexError, AttributeError):
-            return
-        res = []
-        # Flowset record
-        # Now, according to the flow/option data,
-        # let's re-dissect NetflowDataflowsetV9
-        if tid in definitions:
-            tot_len, cls = definitions[tid]
-            while len(data) >= tot_len:
-                res.append(cls(data[:tot_len]))
-                data = data[tot_len:]
-            # Inject dissected data
-            datafl.records = res
-            datafl.do_dissect_payload(data)
-        # Options
-        elif tid in definitions_opts:
-            (scope_len, scope_cls,
-                option_len, option_cls) = definitions_opts[tid]
-            # Dissect scopes
-            if scope_len:
-                res.append(scope_cls(data[:scope_len]))
-            if option_len:
-                res.append(
-                    option_cls(data[scope_len:scope_len + option_len])
-                )
-            if len(data) > scope_len + option_len:
-                res.append(
-                    conf.padding_layer(data[scope_len + option_len:])
-                )
-            # Inject dissected data
-            datafl.records = res
-            datafl.name = "Netflow DataFlowSet V9/10 - OPTIONS"
+        current = pkt
+        while NetflowDataflowsetV9 in current:
+            datafl = current[NetflowDataflowsetV9]
+            tid = datafl.templateID
+            if tid not in definitions and tid not in definitions_opts:
+                ignored.add(tid)
+                return
+            # All data is stored in one record, awaiting to be split
+            # If fieldValue is available, the record has not been
+            # defragmented: pop it
+            try:
+                data = datafl.records[0].fieldValue
+                datafl.records.pop(0)
+            except (IndexError, AttributeError):
+                return
+            res = []
+            # Flowset record
+            # Now, according to the flow/option data,
+            # let's re-dissect NetflowDataflowsetV9
+            if tid in definitions:
+                tot_len, cls = definitions[tid]
+                while len(data) >= tot_len:
+                    res.append(cls(data[:tot_len]))
+                    data = data[tot_len:]
+                # Inject dissected data
+                datafl.records = res
+                if data:
+                    if len(data) <= 4:
+                        datafl.add_payload(conf.padding_layer(data))
+                    else:
+                        datafl.do_dissect_payload(data)
+            # Options
+            elif tid in definitions_opts:
+                (scope_len, scope_cls,
+                    option_len, option_cls) = definitions_opts[tid]
+                # Dissect scopes
+                if scope_len:
+                    res.append(scope_cls(data[:scope_len]))
+                if option_len:
+                    res.append(
+                        option_cls(data[scope_len:scope_len + option_len])
+                    )
+                if len(data) > scope_len + option_len:
+                    res.append(
+                        conf.padding_layer(data[scope_len + option_len:])
+                    )
+                # Inject dissected data
+                datafl.records = res
+                datafl.name = "Netflow DataFlowSet V9/10 - OPTIONS"
+            current = datafl.payload
 
 
 def netflowv9_defragment(plist, verb=1):
@@ -1488,8 +1580,8 @@ class NetflowSession(IPSession):
     """Session used to defragment NetflowV9/10 packets on the flow.
     See help(scapy.layers.netflow) for more infos.
     """
-    def __init__(self, *args):
-        IPSession.__init__(self, *args)
+    def __init__(self, *args, **kwargs):
+        IPSession.__init__(self, *args, **kwargs)
         self.definitions = {}
         self.definitions_opts = {}
         self.ignored = set()
@@ -1520,9 +1612,12 @@ class NetflowOptionsRecordOptionV9(NetflowRecordV9):
 # Aka Set
 class NetflowOptionsFlowsetOptionV9(Packet):
     name = "Netflow Options Template FlowSet V9/10 - Option"
-    fields_desc = [ShortEnumField("optionFieldType", None,
-                                  NetflowV910TemplateFieldTypes),
-                   ShortField("optionFieldlength", 0)]
+    fields_desc = [BitField("enterpriseBit", 0, 1),
+                   BitEnumField("optionFieldType", None, 15,
+                                NetflowV910TemplateFieldTypes),
+                   ShortField("optionFieldlength", 0),
+                   ConditionalField(ShortField("enterpriseNumber", 0),
+                                    lambda p: p.enterpriseBit)]
 
     def default_payload_class(self, p):
         return conf.padding_layer
@@ -1555,24 +1650,21 @@ class NetflowOptionsFlowsetV9(Packet):
                    PacketListField(
                        "options", [],
                        NetflowOptionsFlowsetOptionV9,
-                       length_from=lambda pkt: pkt.option_field_length)]
-
-    def extract_padding(self, s):
-        if self.length is None:
-            return s, ""
-        # Calc pad length
-        pad_len = self.length - self.option_scope_length - \
-            self.option_field_length - 10
-        return s[pad_len:], s[:pad_len]
+                       length_from=lambda pkt: pkt.option_field_length),
+                   StrLenField("pad", None, length_from=lambda pkt: (
+                       pkt.length - pkt.option_scope_length -
+                       pkt.option_field_length - 10))]
 
     def default_payload_class(self, p):
         return conf.padding_layer
 
     def post_build(self, pkt, pay):
-        # Padding 4-bytes with b"\x00"
-        pkt += (-len(pkt) % 4) * b"\x00"
         if self.length is None:
             pkt = pkt[:2] + struct.pack("!H", len(pkt)) + pkt[4:]
+        if self.pad is None:
+            # Padding 4-bytes with b"\x00"
+            start = 10 + self.option_scope_length + self.option_field_length
+            pkt = pkt[:start] + (-len(pkt) % 4) * b"\x00"
         return pkt + pay
 
 
@@ -1603,14 +1695,18 @@ class NetflowOptionsFlowset10(NetflowOptionsFlowsetV9):
                        NetflowOptionsFlowsetOptionV9,
                        count_from=lambda pkt: (
                            pkt.field_count - pkt.scope_field_count
-                       ))]
+                       )),
+                   StrLenField("pad", None, length_from=lambda pkt: (
+                       pkt.length - (pkt.scope_field_count * 4) - 10))]
 
-    def extract_padding(self, s):
+    def post_build(self, pkt, pay):
         if self.length is None:
-            return s, ""
-        # Calc pad length
-        pad_len = self.length - (self.scope_field_count * 4) - 10
-        return s[pad_len:], s[:pad_len]
+            pkt = pkt[:2] + struct.pack("!H", len(pkt)) + pkt[4:]
+        if self.pad is None:
+            # Padding 4-bytes with b"\x00"
+            start = 10 + self.scope_field_count * 4
+            pkt = pkt[:start] + (-len(pkt) % 4) * b"\x00"
+        return pkt + pay
 
 
 bind_layers(NetflowHeader, NetflowHeaderV9, version=9)
